package isrepair

import (
	"sync"
)

/*
	Потокобезопасный признак необходимости восстановления здоровья
*/

// IsRepair -- потокобезопасный признак восстановления здоровья
type IsRepair struct {
	val   bool
	block sync.RWMutex
}

// NewIsRepair -- возвращает новый *IsRepair
func NewIsRepair() *IsRepair {
	return &IsRepair{}
}

// Get -- возвращает хранимое состояние
func (sf *IsRepair) Get() bool {
	sf.block.RLock()
	defer sf.block.RUnlock()
	return sf.val
}

// Set -- устанавливает хранимое состояние
func (sf *IsRepair) Set() {
	sf.block.Lock()
	defer sf.block.Unlock()
	sf.val = true
}

// Reset -- сбрасывает хранимое состояние
func (sf *IsRepair) Reset() {
	sf.block.Lock()
	defer sf.block.Unlock()
	sf.val = false
}
