package shot

import (
	"fmt"
	// "log"
	"strconv"
	"strings"
	"time"

	"wartank/server/serv_bots/warbot/angar/battle/battle_worker/battleon/shot/damage"
	"wartank/server/serv_bots/warbot/angar/battle/battle_worker/battleon/shottime"

	// "wartank/internal/components/sound"
	"wartank/pkg/alias"
	"wartank/pkg/types"
)

/*
	Исходник предоставляет выстрел со свойствами:
		- время до выстрела
		- длительность перезарядки

	Первый параметр постоянно изменяется (после выстрела восстанавливается)
	Второй параметр меняется медленно (в зависимости от количества очков после выстрела)
*/

// Shot -- объект выстрела
type Shot struct {
	types.IBattleOn                    // FIXME!!!
	recharge        *shottime.ShotTime // Сколько времени нужно для полной перезарядки
	damage          *damage.Damage     // Урон от выстрела с памятью
	damageSum       alias.ADamage      // Суммарный урон
	login           string             // Логин для поиска контрольных строк
	chTick          chan int           // Тик для выстрела
}

// NewShot -- возвращает новый *Shot
func NewShot(battle types.IBattleOn, login string) (*Shot, error) {
	{ // Предусловия
		if battle == nil {
			return nil, fmt.Errorf("NewShot(): battle is nil")
		}
		if login == "" {
			return nil, fmt.Errorf("NewShot(): login is empty")
		}
	}
	sf := &Shot{
		IBattleOn: battle,
		recharge:  shottime.NewShotTime(),
		damage:    damage.NewDamage(),
		login:     login,
		chTick:    make(chan int, 2),
	}
	// Установить время перезарядки
	sf.recharge.Set(6800)
	go sf.makeTick()
	go sf.run()
	return sf, nil
}

// Генерирует тики, когда можно стрелять
func (sf *Shot) makeTick() {
	defer func() {
		close(sf.chTick)
		// log._rintf("Shot.makeTick(): сражение завершёно\n")
	}()
	for {
		select {
		case <-sf.Ctx().Done():
			return
		default:
			sf.chTick <- 1 // Первый выстрел, потом спать по таймингу
			recharge := sf.recharge.Get()
			// log._rintf("INFO Shot.run() перезарядка=%v msec\n", recharge)
			// Если идёт перезарядка -- постепенно обнуляем время ожидания
			time.Sleep(time.Millisecond * time.Duration(recharge))
		}
	}
}

// Цикл выстрела (в отдельном потоке)
func (sf *Shot) run() {
	for range sf.chTick {
		if sf.Masking().Get() { // Стрелять нельзя
			continue
		}
		// Стрелять можно, стандартное ожидание
		sf.shot()
		sf.findDamage()
	}
}

// Обновляет возможность выстрела (~)
//
//	Вызывается из отдельного потока
func (sf *Shot) shot() {
	if err := sf.Net().UpdateLst("Сражение+ выстрел"); err != nil { // Проверка на непосредственно битву
		// <span>закончилась 00:00:07 назад</span>
		// log._rintf("ERRO Shot.shot(): при обновлении lstBattleOn, err=\n\t%v\n", err)
		time.Sleep(time.Second * 1)
		return
	}
	var (
		strOut    string
		lstBattle = sf.GetLst()
		isFind    bool
		err       error
	)
	// <a href="pve?6-26.ILinkListener-currentControl-attackRegularShellLink" class="simple-but gray"><span><span>ОБЫЧНЫЕ</span></span></a>
	for _, strOut = range lstBattle {
		if strings.Contains(strOut, `-currentControl-attackRegularShellLink`) {
			isFind = true
			break
		}
	}
	if !isFind {
		// log._rintf("WARN Shot.shot(): не найдены ссылка на выстрел\n")
		time.Sleep(time.Second * 1)
		return
	}
	lstLink := strings.Split(strOut, `<a href="`)
	strLink := lstLink[1]
	lstLink = strings.Split(strLink, `" class="simple-but gray"><span><span>ОБЫЧНЫЕ</span></span></a>`)
	strLink = "http://wartank.ru/" + lstLink[0]
	lstBattle, err = sf.Net().Get(strLink)
	if err != nil {
		// log._rintf("ERRO Shot.shot(): при исполнении GET-команды выстрела обычным снарядом, err=\n\t%v\n", err)
		time.Sleep(time.Second * 1)
		return
	}
	if err = sf.Update(lstBattle); err != nil {
		// log._rintf("ERRO Shot.shot(): при обновлении lstBattle, err=\n\t%v\n", err)
		time.Sleep(time.Second * 1)
		return
	}
	sf.SetNeedManevr()
	// sound.Shot()
}

// Ищет урон от выстрела
func (sf *Shot) findDamage() {
	var (
		ind     = 0
		isFind  = false
		lstShot = sf.GetLst()
		strOut  string
	)

	for ind, strOut = range lstShot {
		// <span class="yellow1 td_u">prospero tank</span>
		if strings.Contains(strOut, `<span class="yellow1 td_u">`+sf.login+`</span>`) {
			ind += 2
			strOut = lstShot[ind]
			isFind = true
			break
		}
	}
	if !isFind { // Возможно был рикошет или манёвр
		// log._rintf("WARN Shot.findDamage(): не найден урон от выстрела, strOut=%q, ind=%v\n", strOut, ind)
		time.Sleep(time.Second * 1)
		return
	}
	lstDamage := strings.Split(strOut, ` на  <span class="red1">`)
	if len(lstDamage) == 1 {
		time.Sleep(time.Second * 1)
		return
	}
	strDamage := lstDamage[1]
	iDamage, err := strconv.Atoi(strDamage)
	if err != nil {
		// log._rintf("ERRO Shot.findDamage(): damage(%v) не число, err=\n\t%v\n", strDamage, err)
		time.Sleep(time.Second * 1)
		return
	}
	if iDamage <= 0 { // Кривой урон от выстрела
		sf.recharge.Dec5()
		// log._rintf("WARN Shot.findDamage(): ошибка в значении урона(%v)\n", iDamage)
		iDamage = 0
	}
	sf.damageSum += alias.ADamage(iDamage)
	// log._rintf("INFO Shot.Damage(): damageSum=%v\n", sf.damageSum)
	if iDamage < 70 {
		sf.damage.Set(alias.ADamage(iDamage))
		sf.recharge.Inc210()
	}
	// log._rintf("INFO Shot.findDamage(): выстрел=+%v, урон=%v", iDamage, sf.damageSum)
	if iDamage == 0 {
		time.Sleep(time.Second * 1)
		return
	}
	sf.setDamage(alias.ADamage(iDamage))
}

// setDamage -- обновляет время перезарядки в зависимости от произведённого урона
func (sf *Shot) setDamage(val alias.ADamage) {
	sf.damage.Set(val)
	switch sf.damage.Result() {
	case "none":
		sf.recharge.Dec5()
	case "up":
		sf.recharge.Dec30()
	case "down":
		sf.recharge.Inc210()
	}
}
